@@ -12,7 +12,7 @@ BEGIN {
   use Exporter   ();
   use AutoLoader qw(AUTOLOAD);
   use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $AUTOLOAD);
-  $VERSION =	 do { my @r=(q$Revision: 1.5 $=~/\d+/g); sprintf "%d."."%02d"x$#r,@r };
+  $VERSION =	 do { my @r=(q$Revision: 1.6 $=~/\d+/g); sprintf "%d."."%02d"x$#r,@r };
   @ISA =         qw();
   @EXPORT =      qw();
   @EXPORT_OK =   qw();
@@ -33,6 +33,7 @@ sub new {
     $fragment->{SIZE} = shift;
     $fragment->{DWO} = shift;
     $fragment->{FILENUMBER} = shift;
+    $fragment->{BLOCKSIZE} = shift;
   }
 
   return $fragment;
@@ -77,6 +78,12 @@ sub driveWrittenOn {
   return $self->{DWO};
 }
 
+sub blockSize {
+  my $self = shift;
+
+  return $self->{BLOCKSIZE};
+}
+
 sub image {
   my $self = shift;
 
